package com.example.cs_fyber_ad

import android.app.Activity
import androidx.fragment.app.Fragment
import android.content.Intent
import androidx.annotation.NonNull
import com.fyber.FairBid
import com.fyber.Fyber
import com.fyber.ads.AdFormat
import com.fyber.fairbid.ads.Banner
import com.fyber.fairbid.ads.ImpressionData
import com.fyber.fairbid.ads.Interstitial
import com.fyber.fairbid.ads.Rewarded
import com.fyber.fairbid.ads.banner.BannerError
import com.fyber.fairbid.ads.banner.BannerListener
import com.fyber.fairbid.ads.interstitial.InterstitialListener
import com.fyber.fairbid.ads.mediation.MediationStartedListener
import com.fyber.fairbid.ads.rewarded.RewardedListener
import com.fyber.fairbid.internal.Logger
import com.fyber.fairbid.user.UserInfo
import com.fyber.requesters.OfferWallRequester
import com.fyber.requesters.RequestCallback
import com.fyber.requesters.RequestError
import io.flutter.Log
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result


/** CsFyberAdPlugin */
class CsFyberAdPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel;
  var activity: Activity? = null;
  var offerwallIntent: Intent? = null;
  var offerwallFyber: Fyber? = null;
  var offerWallRequester: OfferWallRequester? = null;

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "cs_fyber_ad");
    channel.setMethodCallHandler(this);
  }

  fun OnMethodCallHandler(method: String, args: Map<String, Any>?) {
    try {
      Log.i("广告Listener回调", method);
      activity?.runOnUiThread(Runnable { channel.invokeMethod(method!!, args) });
    } catch (e: Exception) {
      Log.e("flutter_fyber", "Error $e");
    }
  }

  fun setupListener() {
    Rewarded.setRewardedListener( object : RewardedListener {
      override fun onShow(var1: String, var2: ImpressionData) {
        OnMethodCallHandler("videoCompleted", mapOf("placementId" to var1, "impressionDataStr" to var2.jsonString));
      }

      override fun onClick(var1: String) {
        OnMethodCallHandler("rewardedClicked", mapOf("placementId" to var1));
      }

      override fun onHide(var1: String) {
        OnMethodCallHandler("rewardedHided", mapOf("placementId" to var1));
      }

      override fun onShowFailure(var1: String, var2: ImpressionData) {
        OnMethodCallHandler("rewardedShowFailure", mapOf("placementId" to var1, "impressionDataStr" to var2.jsonString));
      }

      override fun onAvailable(var1: String) {
        OnMethodCallHandler("rewardedAvailable", mapOf("placementId" to var1));
      }

      override fun onUnavailable(var1: String) {
        OnMethodCallHandler("rewardedUnavailable", mapOf("placementId" to var1));
      }

      override fun onCompletion(var1: String, var2: Boolean) {
        OnMethodCallHandler("rewardedCompletion", mapOf("placementId" to var1, "success" to var2));
      }

      override fun onRequestStart(var1: String) {
        OnMethodCallHandler("rewardedRequestStart", mapOf("placementId" to var1));
      }
    });

    Interstitial.setInterstitialListener(object :InterstitialListener{
      override fun onShow(var1: String, var2: ImpressionData) {
        OnMethodCallHandler("interstitialShowed", mapOf("placementId" to var1, "impressionDataStr" to var2.jsonString));
      }

      override fun onClick(var1: String) {
        OnMethodCallHandler("interstitialClicked", mapOf("placementId" to var1));
      }

      override fun onHide(var1: String) {
        OnMethodCallHandler("interstitialHided", mapOf("placementId" to var1));
      }

      override fun onShowFailure(var1: String, var2: ImpressionData) {
        OnMethodCallHandler("interstitialShowFailure", mapOf("placementId" to var1, "impressionDataStr" to var2.jsonString));
      }

      override fun onAvailable(var1: String) {
        OnMethodCallHandler("interstitialAvailable", mapOf("placementId" to var1));
      }

      override fun onUnavailable(var1: String) {
        OnMethodCallHandler("interstitialUnavailable", mapOf("placementId" to var1));
      }

      override fun onRequestStart(var1: String) {
        OnMethodCallHandler("interstitialRequestStart", mapOf("placementId" to var1));
      }
    });

    Banner.setBannerListener(object: BannerListener{
      override fun onLoad(var1: String) {
        OnMethodCallHandler("bannerLoaded", mapOf("placementId" to var1));
      }

      override fun onShow(var1: String, var2: ImpressionData) {
        OnMethodCallHandler("bannerShowed", mapOf("placementId" to var1, "impressionDataStr" to var2.jsonString));
      }

      override fun onClick(var1: String) {
        OnMethodCallHandler("bannerClicked", mapOf("placementId" to var1));
      }

      override fun onRequestStart(var1: String) {
        OnMethodCallHandler("bannerRequestStart", mapOf("placementId" to var1));
      }

      override fun onError(var1: String, var2: BannerError?) {
        val message = var2?.errorMessage ?: "";
        OnMethodCallHandler("bannerError", mapOf("placementId" to var1, "message" to message));
      }
    });
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    if (call.method == "init") {
      val appId: String = call.argument<String>("appId") ?: return;
//      var fairBid = FairBid(appId);
//      if (fairBid == null) {
//        Log.e("fairBid错误","fairBid 错误 0");
//      } else {
//        Log.i("fairBid成功", "fairBid 成功 0");
//      }
//      fairBid = fairBid.withMediationStartedListener(object : MediationStartedListener {
//        override fun onNetworkStarted(network: com.fyber.fairbid.ads.mediation.MediatedNetwork) {
//          val netVer = network.version ?: "-1";
//          OnMethodCallHandler("mediationNetworkStarted", mapOf("name" to network.name, "version" to netVer));
//        }
//
//        override fun onNetworkFailedToStart(network: com.fyber.fairbid.ads.mediation.MediatedNetwork, errorMessage: kotlin.String) {
//          val netVer = network.version ?: "-1";
//          OnMethodCallHandler("mediationNetworkFailed", mapOf("name" to network.name, "version" to netVer));
//        }
//      });
//
//      if (fairBid == null) {
//        Log.e("fairBid错误","fairBid 错误1");
//      } else {
//        Log.i("fairBid成功", "fairBid 成功 1");
//      }
//      fairBid.start(activity);
//      Log.i("fairBid成功", "fairBid 成功 2");
//      setupListener();

      if (activity != null) {
        offerwallFyber = Fyber.with(appId, activity!!);
        if (offerwallFyber != null) {
        }
      }
      offerWallRequester = OfferWallRequester.create(object : RequestCallback {
        override fun onAdAvailable(intent: Intent) {
          offerwallIntent = intent;
          OnMethodCallHandler("offerWallAvailable", null);
        }

        override fun onAdNotAvailable(adFormat: AdFormat) {
          offerwallIntent = null;
          OnMethodCallHandler("offerWallNotAvailable", mapOf("formatMess" to adFormat.toString()));
        }

        override fun onRequestError(requestError: RequestError) {
          offerwallIntent = null;
          OnMethodCallHandler("offerWallRequestError", mapOf("errorMess" to requestError.description));
        }
      });
      if (offerWallRequester != null) {
        Log.i("Fyber成功", "Fyber 成功 3");
      }
    } else if (call.method == "setDebugLog") {
      val isDebug: Boolean = call.argument<Boolean>("isDebug") ?: return;
      Logger.setDebugLogging(isDebug);
    } else if (call.method == "setUserId") {
      val userId: String = call.argument<String>("userId") ?: return;
      UserInfo.setUserId(userId);
      offerwallFyber = offerwallFyber?.withUserId(userId);
    } else if (call.method == "setupOfferWall") {
      val securityToken: String = call.argument<String>("securityToken") ?: return;
      offerwallFyber = offerwallFyber?.withSecurityToken(securityToken);
      offerwallFyber?.start();
    } else if (call.method == "requestOfferWall") {
      val placementId: String? = call.argument<String>("placementId");
      if (placementId != null) {
        offerWallRequester?.withPlacementId(placementId)?.request(activity);
      } else {
        offerWallRequester?.request(activity);
      }
    } else if (call.method == "showOfferWall") {
      if (offerwallIntent == null ) {
        Log.e("错误","offerwallIntent == null");
        return;
      }

      if (activity == null) {
        Log.e("错误","activity == null");
        return;
      }

      activity!!.startActivityForResult(offerwallIntent,1234);
    } else if (call.method == "requestRewarded") {
      val placementId: String = call.argument<String>("placementId") ?: return;
      Rewarded.request(placementId);
    } else if (call.method == "showRewarded") {
      val placementId: String = call.argument<String>("placementId") ?: return;
      if (activity != null) {
        Rewarded.show(placementId, activity!!);
      }
    } else if (call.method == "rewardedAvailable") {
      val placementId: String = call.argument<String>("placementId") ?: return;
      result.success(Rewarded.isAvailable(placementId));
    } else if (call.method == "requestInterstitial") {
      val placementId: String = call.argument<String>("placementId") ?: return;
      Interstitial.request(placementId);
    } else if (call.method == "showInterstitial") {
      val placementId: String = call.argument<String>("placementId") ?: return;
      if (activity != null) {
        Interstitial.show(placementId, activity!!);
      }
    } else if (call.method == "interstitialAvailable") {
      val placementId: String = call.argument<String>("placementId") ?: return;
      result.success(Interstitial.isAvailable(placementId));
    } else if (call.method == "showBanner") {
      val placementId: String = call.argument<String>("placementId") ?: return;
      if (activity != null) {
        Banner.show(placementId, activity!!);
      }
    } else if (call.method == "destroyBanner") {
      val placementId: String = call.argument<String>("placementId") ?: return;
      Banner.destroy(placementId);
    } else {
      result.notImplemented()
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null);
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = binding.activity;
  }

  override fun onDetachedFromActivityForConfigChanges() {

  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    activity = binding.activity;
  }

  override fun onDetachedFromActivity() {

  }
}
