import 'dart:async';
import 'dart:developer';

import 'package:flutter/services.dart';

typedef MediationNetworkStarted = void Function(String name, String version);
typedef MediationNetworkFailed = void Function(String name, String version, String errorMessage);

typedef OfferWallAvailable = void Function();
typedef OfferWallNotAvailable = void Function(String formatMess);
typedef OfferWallRequestError = void Function(String errorMess);

typedef RewardedShowed = void Function(String placementId, String impressionDataStr);
typedef RewardedClicked = void Function(String placementId);
typedef RewardedHided = void Function(String placementId);
typedef RewardedShowFailure = void Function(String placementId, String impressionDataStr);
typedef RewardedAvailable = void Function(String placementId);
typedef RewardedUnavailable = void Function(String placementId);
typedef RewardedCompletion = void Function(String placementId, bool success);
typedef RewardedRequestStart = void Function(String placementId);

typedef InterstitialShowed = void Function(String placementId, String impressionDataStr);
typedef InterstitialClicked = void Function(String placementId);
typedef InterstitialHided = void Function(String placementId);
typedef InterstitialShowFailure = void Function(String placementId, String impressionDataStr);
typedef InterstitialAvailable = void Function(String placementId);
typedef InterstitialUnavailable = void Function(String placementId);
typedef InterstitialRequestStart = void Function(String placementId);

typedef BannerLoaded = void Function(String placementId);
typedef BannerShowed = void Function(String placementId, String impressionDataStr);
typedef BannerClicked = void Function(String placementId);
typedef BannerRequestStart = void Function(String placementId);
typedef BannerError = void Function(String placementId, String message);

class CsFyberAd {
  static const MethodChannel _channel = MethodChannel('cs_fyber_ad');
  static final CsFyberAd _instance = CsFyberAd._internal();
  factory CsFyberAd() => _instance;
  CsFyberAd._internal() {
    _channel.setMethodCallHandler(_platformCallHandler);
  }

  Future<void> init({required String appId}) async {
    return _channel.invokeMethod("init", <String, dynamic>{
      'appId': appId,
    });
  }

  Future<void> setDebugLog({required bool isDebug}) async {
    return _channel.invokeMethod("setDebugLog", <String, dynamic>{
      'isDebug': isDebug,
    });
  }

  Future<void> setUserId({required String userId}) async {
    return _channel.invokeMethod("setUserId", <String, dynamic>{
      'userId': userId,
    });
  }

  /// 装载积分墙
  Future<void> setupOfferWall({required String securityToken}) async {
    return _channel.invokeMethod("setupOfferWall", <String, dynamic>{
      'securityToken': securityToken,
    });
  }

  /// 请求积分墙
  Future<void> requestOfferWall({required String placementId}) async {
    return _channel.invokeMethod("requestOfferWall", <String, dynamic>{
      'placementId': placementId,
    });
  }

  /// 展示积分墙
  Future<void> showOfferWall() async {
    return _channel.invokeMethod("showOfferWall");
  }

  /// 请求激励视频
  Future<void> requestRewarded({required String placementId}) async {
    return _channel.invokeMethod("requestRewarded", <String, dynamic>{
      'placementId': placementId,
    });
  }

  /// 展示激励视频
  Future<void> showRewarded({required String placementId}) async {
    return _channel.invokeMethod("showRewarded", <String, dynamic>{
      'placementId': placementId,
    });
  }

  /// 激励视频是否准备好
  Future<bool?> rewardedAvailable({required String placementId}) async {
    return _channel.invokeMethod<bool>("rewardedAvailable", <String, dynamic>{
      'placementId': placementId,
    });
  }

  /// 请求插页广告
  Future<void> requestInterstitial({required String placementId}) async {
    return _channel.invokeMethod("requestInterstitial", <String, dynamic>{
      'placementId': placementId,
    });
  }

  /// 展示插页广告
  Future<void> showInterstitial({required String placementId}) async {
    return _channel.invokeMethod("showInterstitial", <String, dynamic>{
      'placementId': placementId,
    });
  }

  /// 插页广告是否准备好
  Future<bool?> interstitialAvailable({required String placementId}) async {
    return _channel.invokeMethod<bool>("interstitialAvailable", <String, dynamic>{
      'placementId': placementId,
    });
  }

  /// 展示横幅广告
  Future<void> showBanner({required String placementId}) async {
    return _channel.invokeMethod("showBanner", <String, dynamic>{
      'placementId': placementId,
    });
  }

  /// 销毁横幅广告
  Future<void> destroyBanner({required String placementId}) async {
    return _channel.invokeMethod("destroyBanner", <String, dynamic>{
      'placementId': placementId,
    });
  }

  late MediationNetworkStarted _mediationNetworkStarted;
  late MediationNetworkFailed _mediationNetworkFailed;

  late OfferWallAvailable _offerWallAvailable;
  late OfferWallNotAvailable _offerWallNotAvailable;
  late OfferWallRequestError _offerWallRequestError;

  late RewardedShowed _rewardedShowed;
  late RewardedClicked _rewardedClicked;
  late RewardedHided _rewardedHided;
  late RewardedShowFailure _rewardedShowFailure;
  late RewardedAvailable _rewardedAvailable;
  late RewardedUnavailable _rewardedUnavailable;
  late RewardedCompletion _rewardedCompletion;
  late RewardedRequestStart _rewardedRequestStart;

  late InterstitialShowed _interstitialShowed;
  late InterstitialClicked _interstitialClicked;
  late InterstitialHided _interstitialHided;
  late InterstitialShowFailure _interstitialShowFailure;
  late InterstitialAvailable _interstitialAvailable;
  late InterstitialUnavailable _interstitialUnavailable;
  late InterstitialRequestStart _interstitialRequestStart;

  late BannerLoaded _bannerLoaded;
  late BannerShowed _bannerShowed;
  late BannerClicked _bannerClicked;
  late BannerRequestStart _bannerRequestStart;
  late BannerError _bannerError;

  Future _platformCallHandler(MethodCall call) async {
    switch (call.method) {
      case "mediationNetworkStarted":
        String name = call.arguments["name"];
        String version = call.arguments["version"];
        _mediationNetworkStarted(name, version);
        break;
      case "mediationNetworkFailed":
        String name = call.arguments["name"];
        String version = call.arguments["version"];
        String errorMessage = call.arguments["errorMessage"];
        _mediationNetworkFailed(name, version, errorMessage);
        break;
      case "offerWallAvailable":
        _offerWallAvailable();
        break;
      case "offerWallNotAvailable":
        String formatMess = call.arguments["formatMess"];
        _offerWallNotAvailable(formatMess);
        break;
      case "offerWallRequestError":
        String errorMess = call.arguments["errorMess"];
        _offerWallRequestError(errorMess);
        break;
      case "rewardedShowed":
        String placementId = call.arguments["placementId"];
        String impressionDataStr = call.arguments["impressionDataStr"];
        _rewardedShowed(placementId, impressionDataStr);
        break;
      case "rewardedClicked":
        String placementId = call.arguments["placementId"];
        _rewardedClicked(placementId);
        break;
      case "rewardedHided":
        String placementId = call.arguments["placementId"];
        _rewardedHided(placementId);
        break;
      case "rewardedShowFailure":
        String placementId = call.arguments["placementId"];
        String impressionDataStr = call.arguments["impressionDataStr"];
        _rewardedShowFailure(placementId, impressionDataStr);
        break;
      case "rewardedAvailable":
        String placementId = call.arguments["placementId"];
        _rewardedAvailable(placementId);
        break;
      case "rewardedUnavailable":
        String placementId = call.arguments["placementId"];
        _rewardedUnavailable(placementId);
        break;
      case "rewardedCompletion":
        String placementId = call.arguments["placementId"];
        bool success = call.arguments["success"];
        _rewardedCompletion(placementId, success);
        break;
      case "rewardedRequestStart":
        String placementId = call.arguments["placementId"];
        _rewardedRequestStart(placementId);
        break;
      case "interstitialShowed":
        String placementId = call.arguments["placementId"];
        String impressionDataStr = call.arguments["impressionDataStr"];
        _interstitialShowed(placementId, impressionDataStr);
        break;
      case "interstitialClicked":
        String placementId = call.arguments["placementId"];
        _interstitialClicked(placementId);
        break;
      case "interstitialHided":
        String placementId = call.arguments["placementId"];
        _interstitialHided(placementId);
        break;
      case "interstitialShowFailure":
        String placementId = call.arguments["placementId"];
        String impressionDataStr = call.arguments["impressionDataStr"];
        _interstitialShowFailure(placementId, impressionDataStr);
        break;
      case "interstitialAvailable":
        String placementId = call.arguments["placementId"];
        _interstitialAvailable(placementId);
        break;
      case "interstitialUnavailable":
        String placementId = call.arguments["placementId"];
        _interstitialUnavailable(placementId);
        break;
      case "interstitialRequestStart":
        String placementId = call.arguments["placementId"];
        _interstitialRequestStart(placementId);
        break;
      case "bannerLoaded":
        String placementId = call.arguments["placementId"];
        _bannerLoaded(placementId);
        break;
      case "bannerShowed":
        String placementId = call.arguments["placementId"];
        String impressionDataStr = call.arguments["impressionDataStr"];
        _bannerShowed(placementId, impressionDataStr);
        break;
      case "bannerClicked":
        String placementId = call.arguments["placementId"];
        _bannerClicked(placementId);
        break;
      case "bannerRequestStart":
        String placementId = call.arguments["placementId"];
        _bannerRequestStart(placementId);
        break;
      case "bannerError":
        String placementId = call.arguments["placementId"];
        String message = call.arguments["message"];
        _bannerError(placementId, message);
        break;
      default:
        log('Unknown method ${call.method} ');
    }
  }

  void setMediationNetworkStarted(MediationNetworkStarted mediationNetworkStarted) => _mediationNetworkStarted = mediationNetworkStarted;
  void setMediationNetworkFailed(MediationNetworkFailed mediationNetworkFailed) => _mediationNetworkFailed = mediationNetworkFailed;
  void setOfferWallAvailable(OfferWallAvailable offerWallAvailable) => _offerWallAvailable = offerWallAvailable;
  void setOfferWallNotAvailable(OfferWallNotAvailable offerWallNotAvailable) => _offerWallNotAvailable = offerWallNotAvailable;
  void setOfferWallRequestError(OfferWallRequestError offerWallRequestError) => _offerWallRequestError = offerWallRequestError;
  void setRewardedShowed(RewardedShowed rewardedShowed) => _rewardedShowed = rewardedShowed;
  void setRewardedClicked(RewardedClicked rewardedClicked) => _rewardedClicked = rewardedClicked;
  void setRewardedHided(RewardedHided rewardedHided) => _rewardedHided = rewardedHided;
  void setRewardedShowFailure(RewardedShowFailure rewardedShowFailure) => _rewardedShowFailure = rewardedShowFailure;
  void setRewardedAvailable(RewardedAvailable rewardedAvailable) => _rewardedAvailable = rewardedAvailable;
  void setRewardedUnavailable(RewardedUnavailable rewardedUnavailable) => _rewardedUnavailable = rewardedUnavailable;
  void setRewardedCompletion(RewardedCompletion rewardedCompletion) => _rewardedCompletion = rewardedCompletion;
  void setRewardedRequestStart(RewardedRequestStart rewardedRequestStart) => _rewardedRequestStart = rewardedRequestStart;
  void setInterstitialShowed(InterstitialShowed interstitialShowed) => _interstitialShowed = interstitialShowed;
  void setInterstitialClicked(InterstitialClicked interstitialClicked) => _interstitialClicked = interstitialClicked;
  void setInterstitialHided(InterstitialHided interstitialHided) => _interstitialHided = interstitialHided;
  void setInterstitialShowFailure(InterstitialShowFailure interstitialShowFailure) => _interstitialShowFailure = interstitialShowFailure;
  void setInterstitialAvailable(InterstitialAvailable interstitialAvailable) => _interstitialAvailable = interstitialAvailable;
  void setInterstitialUnavailable(InterstitialUnavailable interstitialUnavailable) => _interstitialUnavailable = interstitialUnavailable;
  void setInterstitialRequestStart(InterstitialRequestStart interstitialRequestStart) => _interstitialRequestStart = interstitialRequestStart;
  void setBannerLoaded(BannerLoaded bannerLoaded) => _bannerLoaded = bannerLoaded;
  void setBannerShowed(BannerShowed bannerShowed) => _bannerShowed = bannerShowed;
  void setBannerClicked(BannerClicked bannerClicked) => _bannerClicked = bannerClicked;
  void setBannerRequestStart(BannerRequestStart bannerRequestStart) => _bannerRequestStart = bannerRequestStart;
  void setBannerError(BannerError bannerError) => _bannerError = bannerError;
}
